import pytz
import time
from datetime import datetime

from django.core.management.base import BaseCommand
from django.utils import timezone


class Command(BaseCommand):
    help = "Enable/Disabled Maintenance Mode if time is passed"

    def handle(self, *args, **options):
        from django_maintenance_window.models import MaintenanceMode
        config = MaintenanceMode.get_solo()
        now = timezone.now()

        if config.maintenance:
            self.disable_maintenance_mode(config, now)
        else:
            self.enable_maintenance_mode(config, now)

    def datetime_from_utc_to_local(self, utc_datetime):
        now_timestamp = time.time()
        offset = datetime.fromtimestamp(now_timestamp) - datetime.utcfromtimestamp(now_timestamp)
        return utc_datetime + offset


    def enable_maintenance_mode(self, config, now):
        if config.maintenance_from and config.maintenance_from < now:
            config.maintenance = True
            config.maintenance_from = None
            config.save()
            # We do not need to check any further. The maintenance should be enabled.
            return

        for schedule in config.schedules.all():
            unaware = datetime.now().replace(hour=schedule.start_time.hour, minute=schedule.start_time.minute, second=schedule.start_time.second, microsecond=0)
            later = datetime.now().replace(hour=schedule.stop_time.hour, minute=schedule.stop_time.minute, second=schedule.stop_time.second, microsecond=0)
            between = schedule.recurrences.between(unaware, later)
            if between:
                scheduled_date = between[0]
                start = datetime(
                    year=scheduled_date.year,
                    month=scheduled_date.month,
                    day=scheduled_date.day,
                    hour=schedule.start_time.hour,
                    minute=schedule.start_time.minute,
                    second=schedule.start_time.second,
                    tzinfo=pytz.timezone('Europe/Amsterdam')
                )
                tz_now = datetime(
                    year=now.year,
                    month=now.month,
                    day=now.day,
                    hour=now.hour,
                    minute=now.minute,
                    second=now.second,
                    tzinfo=pytz.timezone('Europe/Amsterdam')
                )
                if start < self.datetime_from_utc_to_local(now):
                    config.maintenance = True
                    config.save()
                    return

    def disable_maintenance_mode(self, config, now):
        if config.maintenance_until and config.maintenance_until < now:
            config.maintenance = False
            config.maintenance_until = None
            config.save()
            # We do not need to check any further. The maintenance should be disabled.
            return

        for schedule in config.schedules.all():
            unaware = datetime.now().replace(hour=schedule.start_time.hour, minute=schedule.start_time.minute, second=schedule.start_time.second, microsecond=0)
            later = datetime.now().replace(hour=schedule.stop_time.hour, minute=schedule.stop_time.minute, second=schedule.stop_time.second, microsecond=0)
            between = schedule.recurrences.between(unaware, later)
            if between:
                scheduled_date = between[0]
                end = datetime(
                    year=scheduled_date.year,
                    month=scheduled_date.month,
                    day=scheduled_date.day,
                    hour=schedule.stop_time.hour,
                    minute=schedule.stop_time.minute,
                    second=schedule.stop_time.second,
                )
                if end < self.datetime_from_utc_to_local(now):
                    config.maintenance = True
                    config.save()
                    return



# -*- coding: utf-8 -*-

import django
from django.urls import reverse
from django.shortcuts import render
from django.utils.cache import add_never_cache_headers

from . import settings
from .models import MaintenanceMode


class MaintenanceModeMiddleware:
    """This middleware will check if the maintenance page should be shown."""
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        maintenance_response = self.check_maintenance(request)
        if maintenance_response:
            return maintenance_response

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response

    def check_maintenance(self, request):
        """
        This function will check if maintenance is activated.
        If so return the maintenance page.
        Otherwise continue the request.
        """
        config = MaintenanceMode.get_solo()

        admin_root_url = reverse('admin:index')

        skip_maintenance = False
        if request.path.startswith(admin_root_url) and settings.MAINTENANCE_EXCLUDE_ADMIN_URLS:
            skip_maintenance = True
        if request.user and request.user.is_superuser and settings.MAINTENANCE_EXCLUDE_SUPER_USER:
            skip_maintenance = True
        if request.user and request.user.is_staff and settings.MAINTENANCE_EXCLUDE_STAFF_USER:
            skip_maintenance = True

        if config.maintenance and not skip_maintenance:
            kwargs = {
                'end_date': config.maintenance_until,
                'display_end_date': settings.MAINTENANCE_DISPLAY_END_DATE
            }

            response = render(
                request, settings.MAINTENANCE_TEMPLATE,
                content_type='text/html', status=503, context=kwargs)
            add_never_cache_headers(response)
            return response
        return None



# -*- coding: utf-8 -*-

from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _

from recurrence.fields import RecurrenceField
from solo.models import SingletonModel


class MaintenanceMode(SingletonModel):
    maintenance = models.BooleanField(default=False)
    maintenance_from = models.DateTimeField(null=True, blank=True)
    maintenance_until = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return u"Maintenance Mode"

    class Meta:
        verbose_name = _("Maintenance Mode")

    def clean(self):
        error_message = _('You can not set "maintenance_from" without setting "maintenance_until"')  # noqa
        if self.maintenance_from and not self.maintenance_until:
            raise ValidationError(error_message)

    def save(self, *args, **kwargs):
        self.full_clean()
        return super().save(*args, **kwargs)


class Schedule(models.Model):
    mode = models.ForeignKey(MaintenanceMode, on_delete=models.CASCADE, related_name="schedules")
    recurrences = RecurrenceField()
    start_time = models.TimeField(null=True, blank=True)
    stop_time = models.TimeField(null=True, blank=True)

    def clean(self):
        error_message = _('You can not set "start_time" without setting "stop_time"')
        if self.start_time and not self.stop_time:
            raise ValidationError(error_message)

    def save(self, *args, **kwargs):
        self.full_clean()
        return super().save(*args, **kwargs)



from django.contrib import admin

from solo.admin import SingletonModelAdmin

from .models import MaintenanceMode, Schedule


class ScheduleInlineAdmin(admin.TabularInline):
    model = Schedule
    extra = 1


@admin.register(MaintenanceMode)
class MaintenanceModeAdmin(SingletonModelAdmin):
    inlines = [ScheduleInlineAdmin]



# Generated by Django 3.2 on 2021-08-05 15:00

from django.db import migrations, models
import django.db.models.deletion
import recurrence.fields


class Migration(migrations.Migration):

    dependencies = [
        ('django_maintenance_window', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='maintenancemode',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.CreateModel(
            name='Schedule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('recurrences', recurrence.fields.RecurrenceField()),
                ('start_time', models.TimeField(blank=True, null=True)),
                ('stop_time', models.TimeField(blank=True, null=True)),
                ('mode', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='schedules', to='django_maintenance_window.maintenancemode')),
            ],
        ),
    ]



from django.conf import settings


MAINTENANCE_TEMPLATE = getattr(
    settings, 'MAINTENANCE_TEMPLATE',
    'django_maintenance_window/maintenance.html'
)

MAINTENANCE_DISPLAY_END_DATE = getattr(
    settings, 'MAINTENANCE_DISPLAY_END_DATE', False
)

MAINTENANCE_EXCLUDE_ADMIN_URLS = getattr(
    settings, 'MAINTENANCE_EXCLUDE_ADMIN_URLS', True
)

MAINTENANCE_EXCLUDE_SUPER_USER = getattr(
    settings, 'MAINTENANCE_EXCLUDE_SUPER_USER', False
)

MAINTENANCE_EXCLUDE_STAFF_USER = getattr(
    settings, 'MAINTENANCE_EXCLUDE_STAFF_USER', False
)
